import { BetterSqlite3Adapter } from "@lucia-auth/adapter-sqlite";
import sqlite from "better-sqlite3";
import type { Database as SqLiteConnection } from "better-sqlite3";
import { LuciaAdapter } from "./abstract.js";
import type { IdTokenClaims, TokenSet, UserinfoResponse } from "openid-client";
import type { UserId, Session } from "lucia";
import type { MaybePromise } from "@sveltejs/kit";

class SqliteMemoryLuciaAdapter extends LuciaAdapter<
  Record<never, never>,
  User
> {
  db: SqLiteConnection;

  constructor() {
    const db = sqlite(":memory:");
    super(
      new BetterSqlite3Adapter(db, {
        user: "user",
        session: "session",
      }),
      {
        getUserAttributes(attributes) {
          return {
            subject: attributes.subject,
            claims: JSON.parse(attributes.claims),
          };
        },
      },
    );
    this.db = db;
  }

  initDatabase() {
    this.db.exec(`CREATE TABLE IF NOT EXISTS user (
      id TEXT NOT NULL PRIMARY KEY,
      subject TEXT NOT NULL UNIQUE,
      claims TEXT NOT NULL
    )`);

    this.db.exec(`CREATE TABLE IF NOT EXISTS session (
      id TEXT NOT NULL PRIMARY KEY,
      expires_at INTEGER NOT NULL,
      user_id TEXT NOT NULL,
      id_token TEXT NOT NULL,
      FOREIGN KEY (user_id) REFERENCES user(id)
    )`);
  }

  getUserId(subject: string) {
    const user = this.db
      .prepare("SELECT id FROM user WHERE subject = ?")
      .get(subject) as { id: string } | undefined;
    return user?.id;
  }

  newUser(subject: string, userId: string, claims: IdTokenClaims) {
    this.db
      .prepare("INSERT INTO user (id, subject, claims) VALUES (?, ?, ?)")
      .run(userId, subject, JSON.stringify(claims));
    return userId;
  }

  newSession(
    userId: UserId,
    tokenSet: TokenSet & { id_token: string },
  ): Promise<Session> {
    return this.createSession(userId, {
      idToken: tokenSet.id_token,
    });
  }

  getIdToken(session: Session): MaybePromise<string | undefined> {
    return session.idToken;
  }
}

export const adapter = new SqliteMemoryLuciaAdapter();

interface DatabaseUser {
  id: string;
  subject: string;
  claims: string;
}
interface User {
  subject: string;
  claims: UserinfoResponse;
}
interface DatabaseSession {
  idToken: string;
}

declare module "@arise/aidc-sveltekit" {
  interface Types {
    Session: DatabaseSession;
    User: User;
    // À supprimer si vous copiez-collez ce code
    // @ts-expect-error - Tous les adapters redéfinissent ces attributs
    DatabaseSessionAttributes: DatabaseSession;
    DatabaseUserAttributes: DatabaseUser;
  }
}
